<?php

namespace Drupal\commerceg_product_group\Hook;

use Drupal\commerceg_product_group\Configure\ConfiguratorInterface;
use Drupal\commerceg_product_group\Configure\InstallerInterface;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;

/**
 * Holds methods implementing form alteration hooks.
 */
class FormAlter {

  use StringTranslationTrait;

  /**
   * The product group module configurator.
   *
   * @var \Drupal\commerceg_product_group\Configure\ConfiguratorInterface
   */
  protected $configurator;

  /**
   * The product group module installer.
   *
   * @var \Drupal\commerceg_product_group\Configure\InstallerInterface
   */
  protected $installer;

  /**
   * Constructs a new FormAlter object.
   *
   * @param \Drupal\commerceg_product_group\Configure\ConfiguratorInterface $configurator
   *   The B2B order configurator.
   * @param \Drupal\commerceg_product_group\Configure\InstallerInterface $installer
   *   The B2B order installer.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   */
  public function __construct(
    ConfiguratorInterface $configurator,
    InstallerInterface $installer,
    TranslationInterface $string_translation
  ) {
    $this->configurator = $configurator;
    $this->installer = $installer;
    $this->stringTranslation = $string_translation;
  }

  /**
   * Implements hook_form_FORM_ID_alter().
   *
   * Handles Product Group integration settings for product types. We add a
   * checkbox for enabling/disabling integration for the product type being
   * created/edited.
   *
   * @param array $form
   *   The form render array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function productTypeFormAlter(
    array &$form,
    FormStateInterface $form_state
  ) {
    $product_type = $form_state->getFormObject()->getEntity();

    $description = $this->t(
      'When enabled, products of this type can be added to Product Groups.'
    );

    $disabled = FALSE;
    $can_uninstall = NULL;
    $is_installed = $this->installer->isProductTypeInstalled($product_type);
    if ($is_installed) {
      $can_uninstall = $this->installer->canUninstallProductType($product_type);
    }

    if ($is_installed && !$can_uninstall) {
      $disabled = TRUE;
      $description .= '<p><em>' . $this->t(
        'You cannot remove availability of this product type from Product Groups
         because there already exist products of this type that belong to
         Product Groups.'
      ) . '</em></p>';
    }

    $form['commerceg_product_group_status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Make products available to Product Groups'),
      '#description' => $description,
      '#default_value' => $is_installed,
      '#disabled' => $disabled,
    ];

    // We do not add validate/submit callbacks in this class to avoid mixing
    // static with non-static methods; that would create issues with writing
    // automated tests.
    $form['actions']['submit']['#validate'][] = '\Drupal\commerceg_product_group\Form\Callbacks::productTypeValidate';
    $form['actions']['submit']['#submit'][] = '\Drupal\commerceg_product_group\Form\Callbacks::productTypeSubmit';
  }

}
